@@ -79,6 +79,10 @@ class WeekdayValue:
     month, or -1 represents the last Monday of the month.
     """
 
+    def __str__(self) -> str:
+        """Return the WeekdayValue as an encoded string."""
+        return f"{self.occurrence or ''}{self.weekday}"
+
 
 class Frequency(str, enum.Enum):
     """Type of recurrence rule.
@@ -189,7 +193,8 @@ RRULE_WEEKDAY = {
 WEEKDAY_REGEX = re.compile(r"([-+]?[0-9]*)([A-Z]+)")
 
 RecurInputDict = dict[
-    str, Union[datetime.datetime, str, list[str], list[dict[str, str]]]
+    str,
+    Union[datetime.datetime, datetime.date, str, list[str], list[dict[str, str]], None],
 ]
 
 
@@ -254,7 +259,9 @@ class Recur(BaseModel):
 
     def as_rrule_str(self) -> str:
         """Return the Recur instance as an RRULE string."""
-        return self.__encode_property_value__(self.dict())
+        return self.__encode_property_value__(
+            self.dict(by_alias=True, exclude_none=True, exclude_defaults=True)
+        )
 
     @classmethod
     def from_rrule(cls, rrule_str: str) -> Recur:
@@ -279,22 +286,24 @@ class Recur(BaseModel):
                 value = ",".join([str(val) for val in value])
             elif key == "byday":
                 values = []
-                for weekday_dict in value:
-                    weekday = weekday_dict["weekday"]
-                    occurrence = weekday_dict.get("occurrence")
-                    if occurrence is None:
-                        occurrence = ""
-                    values.append(f"{occurrence}{weekday}")
+                for weekday_value in value:
+                    if isinstance(weekday_value, dict):
+                        weekday_value = WeekdayValue(**weekday_value)
+                    values.append(str(weekday_value))
                 value = ",".join(values)
             elif isinstance(value, datetime.datetime):
                 value = DateTimeEncoder.__encode_property_json__(value)
+            elif isinstance(value, datetime.date):
+                value = DateEncoder.__encode_property_json__(value)
             if not value:
                 continue
             result.append(f"{key.upper()}={value}")
         return ";".join(result)
 
     @classmethod
-    def __parse_property_value__(cls, prop: Any) -> RecurInputDict:
+    def __parse_property_value__(  # pylint: disable=too-many-branches
+        cls, prop: Any
+    ) -> RecurInputDict:
         """Parse the recurrence rule text as a dictionary as Pydantic input.
         An input rule like 'FREQ=YEARLY;BYMONTH=4' is converted
         into dictionary.
@@ -314,9 +323,16 @@ class Recur(BaseModel):
             key, value = part.split("=")
             key = key.lower()
             if key == "until":
-                result[key] = DateTimeEncoder.__parse_property_value__(
-                    ParsedProperty(name="ignored", value=value)
-                )
+                new_value: datetime.datetime | datetime.date | None
+                try:
+                    new_value = DateTimeEncoder.__parse_property_value__(
+                        ParsedProperty(name="ignored", value=value)
+                    )
+                except ValueError:
+                    new_value = DateEncoder.__parse_property_value__(
+                        ParsedProperty(name="ignored", value=value)
+                    )
+                result[key] = new_value
             elif key in ("bymonthday", "bymonth"):
                 result[key] = value.split(",")
             elif key == "byday":
